import pygame
from playercontrol import *
from ball import *
from globals import *
from rules import *
from rain import *
from enemyai import *
from intro import *
from menu import *

pygame.mixer.pre_init(44100,-16,2, 1024 * 3)	
pygame.init()
pygame.font.init()
Global.screen = pygame.display.set_mode([Global.SCREEN_W, Global.SCREEN_H])
pygame.display.set_caption('Magnificent Gunbright')

class Game:
	def __init__(self, level, num_enemy_balls = 50, num_player_balls = 50):
		Global.player_balls = pygame.sprite.RenderUpdates()
		Global.enemy_balls = pygame.sprite.RenderUpdates()
		Global.player = GlowyBall((320, 400), Global.PLAYER, Global.player_balls, num_player_balls)
		Global.enemy = GlowyBall((320, 50), Global.ENEMY, Global.enemy_balls, num_enemy_balls)
		Global.background = pygame.Surface((Global.SCREEN_W, Global.SCREEN_H))
		Global.background.fill([128,128,128])
		Global.player_controller = PlayerController(Global.player)
		Global.rules = Rules()
		Global.rain = RainEngine()
		Global.player_rain = pygame.sprite.RenderUpdates()
		Global.enemy_rain = pygame.sprite.RenderUpdates()
		Global.all = [Global.player_rain, Global.enemy_rain, Global.player_balls, Global.enemy_balls]
		Global.enemy_ai = EnemyAI(Global.enemy, Global.enemy_balls, Global.player, Global.enemy_rain)
		
		font = pygame.font.Font('prstartk.ttf', 18)
		level_txt = font.render('level %d' % level, True, [64, 64, 64])
		Global.background.blit(level_txt, ((Global.SCREEN_W / 2) - (level_txt.get_width() / 2), (Global.SCREEN_H / 2) - (level_txt.get_height() / 2)))

	def pause(self):
		debounce = True
		done = False
		pause_bmp = pygame.image.load('quit.png')
		
		Global.screen.blit(pause_bmp, ((Global.SCREEN_W / 2) - 200, (Global.SCREEN_H / 2) - 50))
		pygame.display.flip()
		
		while not done:
			event_pump()
			keys = pygame.key.get_pressed()
			
			if debounce:
				if not keys[K_ESCAPE]:
					debounce = False
			else:
				if keys[K_ESCAPE]:
					done = True
					debounce = True
				elif keys[K_RETURN]:
					done = True
					Global.state = Global.GAME_MENU
		while debounce:
			event_pump()
			debounce = pygame.key.get_pressed()[K_ESCAPE]

	def play(self):
		endsound = pygame.mixer.Sound('explode.wav')
		Global.screen.blit(Global.background, (0,0))
		pygame.display.flip()
		while Global.state == Global.GAME_PLAY:
			event_pump()

			if pygame.key.get_pressed()[K_ESCAPE]:
				self.pause()
				Global.screen.blit(Global.background, (0,0))
				pygame.display.flip()
				continue
				
			startTime = pygame.time.get_ticks()
	
			Global.rain.tick()
			Global.player_controller.tick()
			Global.enemy_ai.tick()
			Global.rules.tick()
	
			rects = []
			for group in Global.all:
				group.clear(Global.screen, Global.background)
			for group in Global.all:
				group.update()
				rects.extend(group.draw(Global.screen))
			pygame.display.update(rects)
	
			frameTime = pygame.time.get_ticks() - startTime
	
			if (frameTime < 40):
				pygame.time.delay(40 - frameTime)
		
		if (Global.state == Global.GAME_WIN) or (Global.state == Global.GAME_LOSE):
			chan = endsound.play()
			backcolour = 128
			if Global.state == Global.GAME_WIN: 
				backdir = 1
			else:
				backdir = -1
			while chan.get_busy() and Global.state != Global.GAME_END:
				if backcolour > 0 and backcolour < 255:
					backcolour = backcolour + backdir
				Global.background.fill([backcolour, backcolour, backcolour])
				Global.screen.blit(Global.background, (0,0))
				for group in Global.all:
					group.draw(Global.screen)
				pygame.display.flip()
				event_pump()
				pygame.time.delay(40)
			

def set_state(st):
	if (Global.state != Global.GAME_END):
		Global.state = st

while (Global.state != Global.GAME_END):
	if (Global.state == Global.GAME_PLAY):
		Game(level, starting_enemy_balls, starting_player_balls).play()
	elif (Global.state == Global.GAME_MENU):
		(starting_player_balls, starting_enemy_balls) = Menu().play()
		level = 1
	elif (Global.state == Global.GAME_WIN):		
		starting_enemy_balls = starting_enemy_balls + 10
		starting_player_balls = (starting_player_balls * 4) / 5
		level = level + 1
		set_state(Global.GAME_PLAY)
	elif (Global.state == Global.GAME_LOSE):
		set_state(Global.GAME_MENU)
	elif (Global.state == Global.GAME_INTRO):
		Intro().play()
		set_state(Global.GAME_MENU)
		
pygame.display.quit()
